-- Twily 2024 AWESOME -V 4.3 (Not AUR)

-- If LuaRocks is installed, make sure that packages installed through it are
-- found (e.g. lgi). If LuaRocks is not installed, do nothing.
pcall(require, "luarocks.loader")

-- Standard awesome library
local gears = require("gears")
local awful = require("awful")
require("awful.autofocus")
-- Widget and layout library
local wibox = require("wibox")
-- Theme handling library
local beautiful = require("beautiful")
-- Notification library
local naughty = require("naughty")
local menubar = require("menubar")
local hotkeys_popup = require("awful.hotkeys_popup")
-- Enable hotkeys help widget for VIM and other apps
-- when client with a matching name is opened:
require("awful.hotkeys_popup.keys")

-- Load Debian menu entries
local debian = require("debian.menu")
local has_fdo, freedesktop = pcall(require, "freedesktop")

local vicious = require("vicious")
local layouts = require("layouts")


-- Run Once
function run_once(cmd)
        findme = cmd
        firstspace = cmd:find(" ")
        if firstspace then
                findme = cmd:sub(0, firstspace-1)
        end
        awful.util.spawn_with_shell("pgrep -u $USER -c " .. findme .. " > /dev/null || (" .. cmd .. ")")
end

-- setup screen in $ crontab -e 
-- @reboot screen -dmS wallpaper /home/amalie/scripts/display/ana-wall.sh
-- (^ setup in root for sddm wallpaper change as well)
run_once("/home/amalie/scripts/display/ana-wall-next.sh")
--run_once("/home/amalie/scripts/display/xorient.sh landscape")
run_once("/home/amalie/scripts/display/ana-wall-login.sh")
run_once("compton --config /home/amalie/.compton/compton.conf -b")
run_once("bash -c \"sleep 1; echo 'Welcome back Twily!' | dzen2 -w 250 -h 50 -x 1660 -y 40 -p 2\"")
run_once("/home/amalie/startanachrome.sh")
--run_once("urxvt -g 100x25")


local oldspawn = awful.util.spawn
awful.util.spawn = function (s)
    oldspawn(s, false)
end


-- {{{ Error handling
-- Check if awesome encountered an error during startup and fell back to
-- another config (This code will only ever execute for the fallback config)
if awesome.startup_errors then
    naughty.notify({ preset = naughty.config.presets.critical,
                     title = "Oops, there were errors during startup!",
                     text = awesome.startup_errors })
end

-- Handle runtime errors after startup
do
    local in_error = false
    awesome.connect_signal("debug::error", function (err)
        -- Make sure we don't go into an endless error loop
        if in_error then return end
        in_error = true

        naughty.notify({ preset = naughty.config.presets.critical,
                         title = "Oops, an error happened!",
                         text = tostring(err) })
        in_error = false
    end)
end
-- }}}

-- {{{ Variable definitions
-- Themes define colours, icons, font and wallpapers.
--beautiful.init(gears.filesystem.get_themes_dir() .. "default/theme.lua")

-- Variable Definition
home            = os.getenv("HOME")
config_dir         = home .. "/.config/awesome"

beautiful.init(config_dir .. "/theme/theme_new.lua")

--terminal        = "urxvt --depth 32 -bg rgba:0000/0000/0000/6666"
--terminal        = "urxvt --depth 32"
terminal        = "urxvt"
editor          = "vim"
editor_cmd      = terminal .. " -g 100x25 -e " .. editor
gui_editor      = "leafpad"
browser         = "firefox"
tasks           = terminal .. " -g 100x25 -e htop"
volctrl         = "Master"

modkey          = "Mod4"
altkey          = "Mod1"

-- This is used later as the default terminal and editor to run.
--terminal = "x-terminal-emulator"
--editor = os.getenv("EDITOR") or "editor"
--editor_cmd = terminal .. " -e " .. editor

-- Default modkey.
-- Usually, Mod4 is the key with a logo between Control and Alt.
-- If you do not like this or do not have such a key,
-- I suggest you to remap Mod4 to another key using xmodmap or other tools.
-- However, you can use another modifier like Mod1, but it may interact with others.
--modkey = "Mod4"

-- Table of layouts to cover with awful.layout.inc, order matters.
awful.layout.layouts = {
    awful.layout.suit.tile,
    awful.layout.suit.tile.left,
    awful.layout.suit.tile.bottom,
    awful.layout.suit.tile.top,
    awful.layout.suit.magnifier,
    awful.layout.suit.corner.nw,
    awful.layout.suit.floating,
    awful.layout.suit.fair,
    awful.layout.suit.fair.horizontal,
    awful.layout.suit.spiral,
    awful.layout.suit.spiral.dwindle,
    awful.layout.suit.max,
    awful.layout.suit.max.fullscreen,
    ---- awful.layout.suit.corner.ne,
    ---- awful.layout.suit.corner.sw,
    ---- awful.layout.suit.corner.se,
    --awful.layout.suit.floating,
    --layouts.uselesstile_new,
    ----awful.layout.suit.tile,
    --layouts.twily_new
}
-- }}}


-- Tags
tags = {
        --names = { "/main/", "/media/", "/code/", "/misc/" },
        --names = { " ▫ ", " ▣ ", " ▦ ", " ▩ ", " ▪ " },
        names = { " 🬃 ", " 🬑 ", " 🬗 ", " 🬪 ", " 🬺 " },
        --names = { " ✦ ", " ♠ ", " ♣ ", " ♥ ", " ♦ " },
        --names = { " １ ", " ２ ", " ３ ", " ４ ", " ５ " }, -- ０
        --names = { " ₁ ", " ₂ ", " ₃ ", " ₄ ", " ₅ " }, -- ₀
        --names = { " Ⅰ ", " Ⅱ ", " Ⅲ ", " Ⅳ ", " Ⅴ " },
        --names = { " • ", " ∶ ", " ∴ ", " ∷ ", " ∞ " },
        --names = { " ♔ ", " ♕ ", " ♖ ", " ♗ ", " ♘ " }, -- ♙
        --names = { " ⚀ ", " ⚁ ", " ⚂ ", " ⚃ ", " ⚄ " }, -- ⚅
        --names = { " ⋅ ", " ∶ ", " ⋰ ", " ∷ ", " ∞ " },
        --layout = { layouts[3], layouts[2], layouts[2], layouts[2], layouts[2] }
        --layout = { layouts[1], layouts[1], layouts[1], layouts[1], layouts[1] }
}

-- {{{ Menu
-- Create a launcher widget and a main menu
--menu_sep=""
menu_sep="--------------"

-- Menu
myaccessories = {
        { "7-Zip", "7zFM" },
        { "Calculator", "galculator" },
        --{ "Leafpad", gui_editor },
        { "Thunar", "thunar" },
        { menu_sep, "" },
        { "XZoom", "xzoom -mag 2 -geometry 900x200+20-20" }
}
myinternet = {
        { "Firefox", "firefox" },
        { "Chromium", "chromium" },
        { "Vivaldi", "vivaldi" },
        { menu_sep, "" },
        { "Parsec", "parsecd" },
        { "VNC viewer", "vncviewer" },
        { menu_sep, "" },
        { "Thunderbird", "thunderbird" },
        --{ "transmission", "transmission-gtk" },
        { "FileZilla", "filezilla" },
        --{ "Irrsi", terminal .. " -g 100x25 -e irssi" },
        { "rTorrent", terminal .. " -g 100x25 -e rtorrent" }
        --{ "Wicd", terminal .. " -g 100x25 -e wicd-curses" }
}
mymedia = {
        --{ "Ncmpcpp", terminal .. " -g 100x25 -e ncmpcpp" },
        { "Spotify", "spotify" },
        { "VLC", "vlc" },
        { menu_sep, "" },
        { "Volume", terminal .. " -g 100x25 -e alsamixer" }
}
mygames = {
        { "ArmagetronAd", "armagetronad" },
        { "Minetest", "minetest" },
        { "NetHack", terminal .. " -g 100x25 -e nethack" },
        { "Stone-Soup", terminal .. " -g 100x25 -e crawl" }
}
mygraphics = {
        { "Blender", "/home/amalie/scripts/startblender.sh" },
        { menu_sep, "" },
        { "Gimp", "gimp" },
        { "Colors", "gcolor3" },
        { menu_sep, "" },
        { "Feh", "feh /home/amalie/Pictures/" },
        { "Sxiv", "sxiv -tsr /home/amalie/Pictures" }
}
myoffice = {
        { "Libre Base", "libreoffice --base" },
        { "Libre Calc", "libreoffice --calc" },
        { "Libre Draw", "libreoffice --draw" },
        { "Libre Impress", "libreoffice --impress" },
        { "Libre Math", "libreoffice --math" },
        { "Libre Writer", "libreoffice --writer" }
}
mysystem = {
        --{ "GTK-Theme", "gtk-chtheme" }, 
        { "GTK-Theme", "lxappearance" },
        --{ "Lock Screen", "/home/amalie/.lockscreen" },
        { "Process Viewer", tasks },
        { menu_sep, "" },
        { "Logout", "killall awesome" },
        { "Reboot", "reboot" },
        { "Shutdown", "poweroff" },
        { menu_sep, "" },
        { "Performance", "sh /home/amalie/scripts/compton.sh game" },
        { "Quality", "sh /home/amalie/scripts/compton.sh" }
}
myawesomemenu = {
   { "hotkeys", function() hotkeys_popup.show_help(nil, awful.screen.focused()) end },
   { "manual", terminal .. " -e man awesome" },
   { "edit config", editor_cmd .. " " .. awesome.conffile },
   { "restart", awesome.restart },
   { "quit", function() awesome.quit() end },
}

mymainmenu = awful.menu({
        items = {
                { "Ranger", terminal .. " -g 100x25 -e ranger" },
                { "Terminal", terminal .. " -g 100x25" },
                { menu_sep, "" },
                { "Accessories", myaccessories },
                { "Games", mygames },
                { "Graphics", mygraphics },
                { "Internet", myinternet },
                { "Media", mymedia },
                { "Office", myoffice },
                { "System", mysystem },
                { "Awesome", myawesomemenu }
        }
})

local menu_awesome = { "awesome", mymainmenu, beautiful.awesome_icon }
--local menu_terminal = { "open terminal", terminal }
--
--if has_fdo then
--    mymainmenu = freedesktop.menu.build({
--        before = { menu_awesome },
--        after =  { menu_terminal }
--    })
--else
--    mymainmenu = awful.menu({
--        items = {
--                  menu_awesome,
--                  { "Debian", debian.menu.Debian_menu.Debian },
--                  menu_terminal,
--                }
--    })
--end


mylauncher = awful.widget.launcher({ image = beautiful.awesome_icon,
                                     menu = mymainmenu })

-- Menubar configuration
menubar.utils.terminal = terminal -- Set the terminal for applications that require it
-- }}}

-- Keyboard map indicator and switcher
mykeyboardlayout = awful.widget.keyboardlayout()


-- WIBOX

-- Kernel Info
syswidget = wibox.widget.textbox()
vicious.register( syswidget, vicious.widgets.os, "<span color=\"#D3D5E0\">$2</span>" )

-- Uptime
uptimewidget = wibox.widget.textbox()
vicious.register( uptimewidget, vicious.widgets.uptime, "<span color=\"#d6ccdc\">Up</span> <span color=\"#D3D5E0\">$2.$3</span>" )

-- Cpu
cpuwidget = wibox.widget.textbox()
vicious.register( cpuwidget, vicious.widgets.cpu, "<span color=\"#d6ccdc\">Cpu</span> <span color=\"#D3D5E0\">$1%</span>", 3 )

-- Ram
memwidget = wibox.widget.textbox()
vicious.register( memwidget, vicious.widgets.mem, "<span color=\"#d6ccdc\">Mem</span> <span color=\"#D3D5E0\">$2MiB</span>", 5 )

-- Clock
mytextclock = awful.widget.textclock("<span color=\"#aaa5ad\">⮖</span> <span color=\"#d6ccdc\">%a, %d %b %H:%M</span> " )
mytextclock:buttons(awful.util.table.join(
    awful.button({ }, 1, function () awful.util.spawn("sh " .. config_dir .. "/dzen/dzen-cal") end)
))


---- Mpd
--mpdwidget = wibox.widget.textbox()
--vicious.register( mpdwidget, vicious.widgets.mpd, function(widget, args)
--    if args["{state}"] == "Stop" then
--        return ""
--    else
--        return '<span color="#aaa5ad">⮕</span> <span color="#d6ccdc">'..args["{Artist}"]..' - '..args["{Title}"]..'</span>'
--    end
--end, 1)

---- Volume
--volumewidget = wibox.widget.textbox()
--vicious.register(volumewidget, vicious.widgets.volume, function(volumewidget, args)
--    local volbar = ""
--    local volico = "<span color='#aaa5ad'>"
--    local vol = math.floor(args[1] / 10)
--
--    volbar = "<span color='".."#d6ccdc".."'>"..
--        string.rep("⮶",vol)..
--        "</span><span color='".."#4b464f".."'>"..
--        string.rep("⮶",10-vol).."</span>"
--
--    if args[2] == "♩" then
--        volico = volico.."⮝"
--    else
--        if args[1] > 20 then
--            volico = volico.."⮞"
--        else
--            volico = volico.."⮟"
--        end
--    end
--    volico = volico.."</span>"
--
--    return volico..' '..volbar
--end, 1, volctrl )
--volumewidget:buttons(awful.util.table.join(
--    awful.button({ }, 1, function () awful.util.spawn("sh " .. config_dir .. "/dzen/dzen-vol") end)
--))

-- Battery
batwidget = wibox.widget.textbox()
vicious.register(batwidget, vicious.widgets.bat, function (batwidget, args)
    local batbar = ""
    local batico = "<span color='#aaa5ad'>"
    local bat = math.floor(args[2] / 10)

    batbar = "<span color='".."#d6ccdc".."'>"..
        string.rep("⮶",bat)..
        "</span><span color='".."#4b464f".."'>"..
        string.rep("⮶",10-bat).."</span>"
    if args[1] == "↯" then
        batico = batico.."⮎"
    elseif args[1] == "+" then
        batico = batico.."⮒"
    elseif args[1] == "-" then
        if args[2] > 60 then
            batico = batico.."⮏"
        elseif args[2] > 10 then
            batico = batico.."⮑"
        else
            batico = batico.."⮐"
        end
    end
    batico = batico.."</span>"

    return batico..' '..batbar
end, 5, "TID2" )
batwidget:buttons(awful.util.table.join(
    awful.button({ }, 1, function () awful.util.spawn("sh " .. config_dir .. "/dzen/dzen-bat") end)
)) -- TID2 = /sys/class/power_supply/TID2/capacity





-- + MPD + Volume + Battery skipped twily +
--
-- ..

-- {{{ Wibar
-- Create a textclock widget
--mytextclock = wibox.widget.textclock()


-- Spacer
space = wibox.widget.textbox()
space:set_text(' ')

-- Sepearator
sepa = wibox.widget.textbox()
sepa:set_text('   ')

separator = wibox.widget.imagebox()
separator:set_image(awful.util.getdir("config") .. "/theme/separator.png")


-- Create a wibox for each screen and add it
local taglist_buttons = gears.table.join(
                    awful.button({ }, 1, function(t) t:view_only() end),
                    awful.button({ modkey }, 1, function(t)
                                              if client.focus then
                                                  client.focus:move_to_tag(t)
                                              end
                                          end),
                    awful.button({ }, 3, awful.tag.viewtoggle),
                    awful.button({ modkey }, 3, function(t)
                                              if client.focus then
                                                  client.focus:toggle_tag(t)
                                              end
                                          end),
                    awful.button({ }, 4, function(t) awful.tag.viewnext(t.screen) end),
                    awful.button({ }, 5, function(t) awful.tag.viewprev(t.screen) end)
                )

local tasklist_buttons = gears.table.join(
                     awful.button({ }, 1, function (c)
                                              if c == client.focus then
                                                  c.minimized = true
                                              else
                                                  c:emit_signal(
                                                      "request::activate",
                                                      "tasklist",
                                                      {raise = true}
                                                  )
                                              end
                                          end),
                     awful.button({ }, 3, function()
                                              awful.menu.client_list({ theme = { width = 250 } })
                                          end),
                     awful.button({ }, 4, function ()
                                              awful.client.focus.byidx(1)
                                          end),
                     awful.button({ }, 5, function ()
                                              awful.client.focus.byidx(-1)
                                          end))

local function set_wallpaper(s)
    -- Wallpaper
    if beautiful.wallpaper then
        local wallpaper = beautiful.wallpaper
        -- If wallpaper is a function, call it with the screen
        if type(wallpaper) == "function" then
            wallpaper = wallpaper(s)
        end
        gears.wallpaper.maximized(wallpaper, s, true)
    else
        -- fallback or windows will appear frozen/not closing become part of background
        gears.wallpaper.set("#000011")
    end
end

-- Re-set wallpaper when a screen's geometry changes (e.g. different resolution)
screen.connect_signal("property::geometry", set_wallpaper)


awful.screen.connect_for_each_screen(function(s)
    -- Wallpaper
    set_wallpaper(s)

    -- Each screen has its own tag table.
    --awful.tag({ "1", "2", "3", "4", "5", "6", "7", "8", "9" }, s, awful.layout.layouts[1])
    awful.tag(tags.names, s, awful.layout.layouts[1])

    -- Create a promptbox for each screen
    s.mypromptbox = awful.widget.prompt()
    -- Create an imagebox widget which will contain an icon indicating which layout we're using.
    -- We need one layoutbox per screen.
    s.mylayoutbox = awful.widget.layoutbox(s)
    s.mylayoutbox:buttons(gears.table.join(
                           awful.button({ }, 1, function () awful.layout.inc( 1) end),
                           awful.button({ }, 3, function () awful.layout.inc(-1) end),
                           awful.button({ }, 4, function () awful.layout.inc( 1) end),
                           awful.button({ }, 5, function () awful.layout.inc(-1) end)))
    -- Create a taglist widget
    s.mytaglist = awful.widget.taglist {
        screen  = s,
        filter  = awful.widget.taglist.filter.all,
        buttons = taglist_buttons
    }

    -- Create a tasklist widget
    s.mytasklist = awful.widget.tasklist {
        screen  = s,
        filter  = awful.widget.tasklist.filter.currenttags,
        buttons = tasklist_buttons
    }

    -- Create the wibox
    s.mywibox = awful.wibar({ position = "top", screen = s })

    -- Add widgets to the wibox
    s.mywibox:setup {
        layout = wibox.layout.align.horizontal,
        { -- Left widgets
            layout = wibox.layout.fixed.horizontal,
            --sepa,
            s.mytaglist,
            mylauncher,
            s.mypromptbox,
            sepa,
        },
        s.mytasklist, -- Middle widget
        { -- Right widgets
            layout = wibox.layout.fixed.horizontal,
            mykeyboardlayout,
            wibox.widget.systray(),
            --volumewidget,
            sepa,
            batwidget,
            mytextclock,
            s.mylayoutbox,
        },
    }
end)
-- }}}

-- {{{ Mouse bindings
root.buttons(gears.table.join(
    awful.button({ }, 3, function () mymainmenu:toggle() end),
    awful.button({ }, 4, awful.tag.viewprev),
    awful.button({ }, 5, awful.tag.viewnext) -- flip for scroll tag direction
))
-- }}}

-- {{{ Key bindings
globalkeys = gears.table.join(
    awful.key({ modkey,           }, "s",      hotkeys_popup.show_help,
              {description="show help", group="awesome"}),
    awful.key({ modkey,           }, "Left",   awful.tag.viewprev,
              {description = "view previous", group = "tag"}),
    awful.key({ modkey,           }, "Right",  awful.tag.viewnext,
              {description = "view next", group = "tag"}),
    awful.key({ modkey,           }, "Escape", awful.tag.history.restore,
              {description = "go back", group = "tag"}),

    awful.key({ modkey,           }, "j",
        function ()
            awful.client.focus.byidx( 1)
        end,
        {description = "focus next by index", group = "client"}
    ),
    awful.key({ modkey,           }, "k",
        function ()
            awful.client.focus.byidx(-1)
        end,
        {description = "focus previous by index", group = "client"}
    ),
    awful.key({ modkey,           }, "w", function () mymainmenu:show() end,
              {description = "show main menu", group = "awesome"}),

    -- Layout manipulation
    awful.key({ modkey, "Shift"   }, "j", function () awful.client.swap.byidx(  1)    end,
              {description = "swap with next client by index", group = "client"}),
    awful.key({ modkey, "Shift"   }, "k", function () awful.client.swap.byidx( -1)    end,
              {description = "swap with previous client by index", group = "client"}),
    awful.key({ modkey, "Control" }, "j", function () awful.screen.focus_relative( 1) end,
              {description = "focus the next screen", group = "screen"}),
    awful.key({ modkey, "Control" }, "k", function () awful.screen.focus_relative(-1) end,
              {description = "focus the previous screen", group = "screen"}),
    awful.key({ modkey,           }, "u", awful.client.urgent.jumpto,
              {description = "jump to urgent client", group = "client"}),
    awful.key({ modkey,           }, "Tab",
        function ()
            awful.client.focus.history.previous()
            if client.focus then
                client.focus:raise()
            end
        end,
        {description = "go back", group = "client"}),

    -- Standard program
    --
    awful.key({ modkey, "Control" }, "c",     function () awful.util.spawn_with_shell("kill -9 " .. get_xproperty("_NET_WM_PID(CARDINAL)")) end),
    awful.key({ modkey, "Shift" }, "p",       function () awful.util.spawn_with_shell("sh /home/amalie/scripts/display/screenshot.sh") end),
    awful.key({ modkey, "Shift" }, "y",       function () awful.util.spawn_with_shell("xzoom -mag 2 -geometry 900x200+20-20") end),
    --
    awful.key({ modkey,           }, "Return", function () awful.spawn(terminal) end,
              {description = "open a terminal", group = "launcher"}),
    awful.key({ modkey,           }, "e", function () awful.spawn("thunar") end,
              {description = "open a filebrowser", group = "launcher"}),
    awful.key({ modkey, "Control" }, "r", awesome.restart,
              {description = "reload awesome", group = "awesome"}),
    --awful.key({ modkey, "Shift"   }, "q", awesome.quit,
    --          {description = "quit awesome", group = "awesome"}),

    awful.key({ modkey,           }, "l",     function () awful.tag.incmwfact( 0.05)          end,
              {description = "increase master width factor", group = "layout"}),
    awful.key({ modkey,           }, "h",     function () awful.tag.incmwfact(-0.05)          end,
              {description = "decrease master width factor", group = "layout"}),
    awful.key({ modkey, "Shift"   }, "h",     function () awful.tag.incnmaster( 1, nil, true) end,
              {description = "increase the number of master clients", group = "layout"}),
    awful.key({ modkey, "Shift"   }, "l",     function () awful.tag.incnmaster(-1, nil, true) end,
              {description = "decrease the number of master clients", group = "layout"}),
    awful.key({ modkey, "Control" }, "h",     function () awful.tag.incncol( 1, nil, true)    end,
              {description = "increase the number of columns", group = "layout"}),
    awful.key({ modkey, "Control" }, "l",     function () awful.tag.incncol(-1, nil, true)    end,
              {description = "decrease the number of columns", group = "layout"}),
    awful.key({ modkey,           }, "space", function () awful.layout.inc( 1)                end,
              {description = "select next", group = "layout"}),
    awful.key({ modkey, "Shift"   }, "space", function () awful.layout.inc(-1)                end,
              {description = "select previous", group = "layout"}),

    awful.key({ modkey, "Control" }, "n",
              function ()
                  local c = awful.client.restore()
                  -- Focus restored client
                  if c then
                    c:emit_signal(
                        "request::activate", "key.unminimize", {raise = true}
                    )
                  end
              end,
              {description = "restore minimized", group = "client"}),

    -- Prompt
    awful.key({ modkey },            "r",     function () awful.screen.focused().mypromptbox:run() end,
              {description = "run prompt", group = "launcher"}),

    awful.key({ modkey }, "x",
              function ()
                  awful.prompt.run {
                    prompt       = "Run Lua code: ",
                    textbox      = awful.screen.focused().mypromptbox.widget,
                    exe_callback = awful.util.eval,
                    history_path = awful.util.get_cache_dir() .. "/history_eval"
                  }
              end,
              {description = "lua execute prompt", group = "awesome"}),
    -- Menubar
    awful.key({ modkey }, "p", function() menubar.show() end,
              {description = "show the menubar", group = "launcher"})
)

clientkeys = gears.table.join(
    awful.key({ modkey,           }, "f",
        function (c)
            c.fullscreen = not c.fullscreen
            c:raise()
        end,
        {description = "toggle fullscreen", group = "client"}),
    awful.key({ modkey, "Shift"   }, "c",      function (c) c:kill()                         end,
              {description = "close", group = "client"}),
    awful.key({ modkey, "Control" }, "space",  awful.client.floating.toggle                     ,
              {description = "toggle floating", group = "client"}),
    awful.key({ modkey, "Control" }, "Return", function (c) c:swap(awful.client.getmaster()) end,
              {description = "move to master", group = "client"}),
    awful.key({ modkey,           }, "o",      function (c) c:move_to_screen()               end,
              {description = "move to screen", group = "client"}),
    awful.key({ modkey,           }, "t",      function (c) c.ontop = not c.ontop            end,
              {description = "toggle keep on top", group = "client"}),
    awful.key({ modkey,           }, "n",
        function (c)
            -- The client currently has the input focus, so it cannot be
            -- minimized, since minimized clients can't have the focus.
            c.minimized = true
        end ,
        {description = "minimize", group = "client"}),
    awful.key({ modkey,           }, "m",
        function (c)
            c.maximized = not c.maximized
            c:raise()
        end ,
        {description = "(un)maximize", group = "client"}),
    awful.key({ modkey, "Control" }, "m",
        function (c)
            c.maximized_vertical = not c.maximized_vertical
            c:raise()
        end ,
        {description = "(un)maximize vertically", group = "client"}),
    awful.key({ modkey, "Shift"   }, "m",
        function (c)
            c.maximized_horizontal = not c.maximized_horizontal
            c:raise()
        end ,
        {description = "(un)maximize horizontally", group = "client"})
)

-- Bind all key numbers to tags.
-- Be careful: we use keycodes to make it work on any keyboard layout.
-- This should map on the top row of your keyboard, usually 1 to 9.
for i = 1, 9 do
    globalkeys = gears.table.join(globalkeys,
        -- View tag only.
        awful.key({ modkey }, "#" .. i + 9,
                  function ()
                        local screen = awful.screen.focused()
                        local tag = screen.tags[i]
                        if tag then
                           tag:view_only()
                        end
                  end,
                  {description = "view tag #"..i, group = "tag"}),
        -- Toggle tag display.
        awful.key({ modkey, "Control" }, "#" .. i + 9,
                  function ()
                      local screen = awful.screen.focused()
                      local tag = screen.tags[i]
                      if tag then
                         awful.tag.viewtoggle(tag)
                      end
                  end,
                  {description = "toggle tag #" .. i, group = "tag"}),
        -- Move client to tag.
        awful.key({ modkey, "Shift" }, "#" .. i + 9,
                  function ()
                      if client.focus then
                          local tag = client.focus.screen.tags[i]
                          if tag then
                              client.focus:move_to_tag(tag)
                          end
                     end
                  end,
                  {description = "move focused client to tag #"..i, group = "tag"}),
        -- Toggle tag on focused client.
        awful.key({ modkey, "Control", "Shift" }, "#" .. i + 9,
                  function ()
                      if client.focus then
                          local tag = client.focus.screen.tags[i]
                          if tag then
                              client.focus:toggle_tag(tag)
                          end
                      end
                  end,
                  {description = "toggle focused client on tag #" .. i, group = "tag"})
    )
end

clientbuttons = gears.table.join(
    awful.button({ }, 1, function (c)
        c:emit_signal("request::activate", "mouse_click", {raise = true})
    end),
    awful.button({ modkey }, 1, function (c)
        c:emit_signal("request::activate", "mouse_click", {raise = true})
        awful.mouse.client.move(c)
    end),
    awful.button({ modkey }, 3, function (c)
        c:emit_signal("request::activate", "mouse_click", {raise = true})
        awful.mouse.client.resize(c)
    end)
)

-- Set keys
root.keys(globalkeys)
-- }}}

-- {{{ Rules
-- Rules to apply to new clients (through the "manage" signal).
awful.rules.rules = {
    -- All clients will match this rule.
    { rule = { },
      properties = { border_width = beautiful.border_width,
                     border_color = beautiful.border_normal,
                     focus = awful.client.focus.filter,
                     raise = true,
                     keys = clientkeys,
                     buttons = clientbuttons,
                     screen = awful.screen.preferred,
                     placement = awful.placement.no_overlap+awful.placement.no_offscreen
     }
    },

    -- Floating clients.
    { rule_any = {
        instance = {
          "DTA",  -- Firefox addon DownThemAll.
          "copyq",  -- Includes session name in class.
          "pinentry",
        },
        class = {
          "Arandr",
          "Blueman-manager",
          "Gpick",
          "Kruler",
          "MessageWin",  -- kalarm.
          "Sxiv",
          "Tor Browser", -- Needs a fixed window size to avoid fingerprinting by screen size.
          "Wpa_gui",
          "veromix",
          "vmg",
          "xtightvncviewer"},

        -- Note that the name property shown in xprop might be set slightly after creation of the client
        -- and the name shown there might not match defined rules here.
        name = {
          "Event Tester",  -- xev.
          "xzoom*"
        },
        role = {
          "AlarmWindow",  -- Thunderbird's calendar.
          "ConfigManager",  -- Thunderbird's about:config.
          "pop-up",       -- e.g. Google Chrome's (detached) Developer Tools.
        }
      }, properties = { floating = true }},
                    -- vmg magnifyer 3.7, couldnt get this to work
                    -- use xzoom instead with awesome
    { rule_any = {
        class = {
          "vmg"
        },
        name = {
          "xzoom*"
        }
      }, properties = { ontop = true }
    },

    -- Add titlebars to normal clients and dialogs
    { rule_any = {type = { "normal", "dialog" }
      }, properties = { titlebars_enabled = true }
    },

    -- Set Firefox to always map on the tag named "2" on screen 1.
    -- { rule = { class = "Firefox" },
    --   properties = { screen = 1, tag = "2" } },
}
-- }}}

-- {{{ Signals
-- Signal function to execute when a new client appears.
client.connect_signal("manage", function (c)
    -- Set the windows at the slave,
    -- i.e. put it at the end of others instead of setting it master.
    -- if not awesome.startup then awful.client.setslave(c) end

    if awesome.startup
      and not c.size_hints.user_position
      and not c.size_hints.program_position then
        -- Prevent clients from being unreachable after screen count changes.
        awful.placement.no_offscreen(c)
    end
end)

-- Add a titlebar if titlebars_enabled is set to true in the rules.
client.connect_signal("request::titlebars", function(c)
    -- buttons for the titlebar
    local buttons = gears.table.join(
        awful.button({ }, 1, function()
            c:emit_signal("request::activate", "titlebar", {raise = true})
            awful.mouse.client.move(c)
        end),
        awful.button({ }, 3, function()
            c:emit_signal("request::activate", "titlebar", {raise = true})
            awful.mouse.client.resize(c)
        end)
    )

    awful.titlebar(c) : setup {
        { -- Left
            awful.titlebar.widget.iconwidget(c),
            buttons = buttons,
            layout  = wibox.layout.fixed.horizontal
        },
        { -- Middle
            { -- Title
                align  = "center",
                widget = awful.titlebar.widget.titlewidget(c)
            },
            buttons = buttons,
            layout  = wibox.layout.flex.horizontal
        },
        { -- Right
            awful.titlebar.widget.floatingbutton (c),
            awful.titlebar.widget.maximizedbutton(c),
            awful.titlebar.widget.stickybutton   (c),
            awful.titlebar.widget.ontopbutton    (c),
            awful.titlebar.widget.closebutton    (c),
            layout = wibox.layout.fixed.horizontal()
        },
        layout = wibox.layout.align.horizontal
    }
end)

-- Enable sloppy focus, so that focus follows mouse.
client.connect_signal("mouse::enter", function(c)
    c:emit_signal("request::activate", "mouse_enter", {raise = false})
end)

client.connect_signal("focus", function(c) c.border_color = beautiful.border_focus end)
client.connect_signal("unfocus", function(c) c.border_color = beautiful.border_normal end)
-- }}}
